//*********************************************************************
//* SPI Test for a SPI connected SRam on a Raspberry
//*********************************************************************
//* Description :
//* This Program should communicate with SPI to a static ram
//* The RAM has a 32,768 x 8-bit Organization and 32-Byte Pages.
//*
//********************************************************************
//* Used unit test_spi from http://wiki.freepascal.org/Raspberry_Pi_-_SPI
//* Written on a Raspberry 3 (stretch)
//* Lazarus : v1.6.2
//* Autor   : M.\\/eber  (only made the IDE)
//* Version : 1.00  Date : 03.11.2017
//***********************************************************************
//* How to use :
//* Use a SRAM 23K256 (Microchip).
//* Connect Pin 1 1/CS (Chip Select Input) to GPIO8 (CE0) oder GPIO7 (CE1)
//* Connect Pin 2 SO (Serial Data Output) to GPIO 9 (MISO)
//* Do not connect Pin 3 NC
//* Connect Pin 4 VSS to Ground
//* Connect Pin 5 SI (Serial Data Input) to GPIO 10 (MOSI)
//* Connect Pin 6 SCK (Serial Clock Input) to GPIO 11 (SCLK)
//* Connect Pin 7 1/Hold (Hold Input) to 3.3 VDC
//* Connect Pin 8 VCC () to 3.3 VDC

//*************************************************************************
//* Additional Info :
//* Datasheet : http://ww1.microchip.com/downloads/en/DeviceDoc/22100F.pdf


unit Unit1;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, FileUtil, Forms, Controls, Graphics, Dialogs, StdCtrls,
  Spin, EditBtn, ExtCtrls, test_spi, strutils;

type

  { TForm1 }

  TForm1 = class(TForm)
    Bevel1: TBevel;
    Bevel2: TBevel;
    Bevel3: TBevel;
    btnDataIn: TButton;
    btnDataOut: TButton;
    btnSendReceive: TButton;
    btnSendOnly: TButton;
    btnSingleByte: TButton;
    btnPageWrite: TButton;
    btnByteWrite: TButton;
    btnByteRead: TButton;
    btnSetStatusSequential: TButton;
    btnReadStatus: TButton;
    btnSetStatusPage: TButton;
    btnSetStatusByte: TButton;
    btnPageRead: TButton;
    btnSeqWrite: TButton;
    btnSeqRead: TButton;
    btnByteFill: TButton;
    btnPageFill: TButton;
    btnShowPage: TButton;
    edtByteValue: TEdit;
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    Label5: TLabel;
    Label6: TLabel;
    Label7: TLabel;
    Label8: TLabel;
    lblPageNr: TLabel;
    Memo1: TMemo;
    rbtnSPIDev0: TRadioButton;
    rbtnSPIDev1: TRadioButton;
    spnedtAddressBrowse: TSpinEdit;
    spnedtByteValue2: TSpinEdit;
    spnedtAddress: TSpinEdit;
    spnedtByteValue: TSpinEdit;
    spnedtPageNr: TSpinEdit;
    procedure btnByteFillClick(Sender: TObject);
    procedure btnByteReadClick(Sender: TObject);
    procedure btnByteWriteClick(Sender: TObject);
    procedure btnDataInClick(Sender: TObject);
    procedure btnDataOutClick(Sender: TObject);
    procedure btnPageFillClick(Sender: TObject);
    procedure btnPageReadClick(Sender: TObject);
    procedure btnPageWriteClick(Sender: TObject);
    procedure btnReadStatusClick(Sender: TObject);
    procedure btnSendOnlyClick(Sender: TObject);
    procedure btnSendReceiveClick(Sender: TObject);
    procedure btnSeqReadClick(Sender: TObject);
    procedure btnSeqWriteClick(Sender: TObject);
    procedure btnSetStatusByteClick(Sender: TObject);
    procedure btnShowPageClick(Sender: TObject);
    procedure btnSingleByteClick(Sender: TObject);
    procedure btnSetStatusSequentialClick(Sender: TObject);
    procedure btnSetStatusPageClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure rbtnSPIDev0Change(Sender: TObject);
    procedure rbtnSPIDev1Change(Sender: TObject);
    procedure spnedtAddressBrowseChange(Sender: TObject);
    procedure spnedtAddressChange(Sender: TObject);
    procedure spnedtByteValue2Change(Sender: TObject);
    procedure spnedtByteValueChange(Sender: TObject);
    procedure spnedtPageNrChange(Sender: TObject);
  private
    { private declarations }
  public
    { public declarations }
  end;

const
  Instr_Read  = $03;
  Instr_Write = $02;
  Instr_RDSR  = $05;
  Instr_WRSR  = $01;


var
  Form1: TForm1;
  SRamAddr : Word;
  SRamDataByte : Byte;
  PageNum : Word;


implementation

{$R *.lfm}

{ TForm1 }

//**********************************************************
//* Procedure : Data in (Buffer Read)
//**********************************************************
procedure TForm1.btnDataInClick(Sender: TObject);
Var
  DataInValue : Integer;
  Din, cnt : Byte;
begin
  Din := 1;
  Cnt := 1;
  DataInValue := spi.DataIn(din,cnt);
  Memo1.lines.add('Data In : '+IntToStr(DataInValue));

end;

//**********************************************************
//* Procedure : Data Out (Buffer Write)
//**********************************************************
procedure TForm1.btnDataOutClick(Sender: TObject);    // Buffer Write
Var
  Result : Integer;
  Dout, cnt : Byte;
  OutArr : Array[0..4] of Byte;
begin
  OutArr[0] := Instr_Write ;
  OutArr[1] := 66;
  OutArr[2] := 34;
  OutArr[3] := 133;

  dout := 9;
  cnt := 4;
  Result := spi.DataOut(OutArr,cnt);
  // Result := spi.DataOut(dout,cnt);
end;





//**********************************************************
//* Procedure : Data Send And Receive
//**********************************************************
procedure TForm1.btnSendReceiveClick(Sender: TObject);
Var
  dout,din,len : Byte;
begin
  spi.TransferSync(dout,din,len);
end;





//**********************************************************
//* Procedure : Data Send Only
//**********************************************************
procedure TForm1.btnSendOnlyClick(Sender: TObject);
// Not clear now  what it does, if i have use  for that on SRAM Project
Var
  dout,len : Byte;
begin
  Memo1.lines.add('Proc : Send Data Only');

  dout := %00110011;  // This initializes only the first byte
 // len := 5;

  Memo1.lines.add('Data Out : '+IntToStr(Dout));
  Memo1.lines.add('Data Len : '+IntToStr(Len));
  spi.TransmitSync(dout,len);
end;

//**********************************************************
//* Procedure : Receive Single Byte
//**********************************************************
procedure TForm1.btnSingleByteClick(Sender: TObject);
// Not clear now  what it does, if i have use  for that on SRAM Project
Var
  ReceiveByte : Byte;
  dout : Byte;
begin
  ReceiveByte := spi.FastShift(dout);
  Memo1.lines.add('Data In : '+IntToStr(ReceiveByte));
end;

//**********************************************************
//* Procedure : Byte Write (seems to work correct)
//**********************************************************
procedure TForm1.btnByteWriteClick(Sender: TObject);
Var
  ByteCount  :  Byte;    // Nr of Bytes to transmit
  OutArr : Array[0..3] of Byte;
begin
  Memo1.lines.add('Proc : Byte Write');
  btnSetStatusByte.Click;

  OutArr[0] := %00000010;   //Instruction
  OutArr[1] := Hi(SRamAddr);   // MSB  A15-A8
  OutArr[2] := Lo(SRamAddr);   // LSB  A7-A0
  OutArr[3] := spnedtByteValue.value;   // Data to store
  ByteCount    := 4;
  spi.TransmitSync(OutArr, ByteCount);
  Memo1.lines.add('Stored Byte : '+IntToStr(SRAMDataByte)+' to '+IntToStr(SRamAddr));
end;

//**********************************************************
//* Procedure : Byte Read (seems to work correct)
//**********************************************************
procedure TForm1.btnByteReadClick(Sender: TObject);
Var
  ByteCnt :  Byte;                 // Nr of Bytes to transmit
  OutArr  : Array[0..3] of Byte;
  InArr   : Array[0..3] of Byte;

begin
  //Memo1.lines.add('Proc : Byte Read');
  btnSetStatusByte.Click;

  spnedtAddress.value;


  OutArr[0] :=  Instr_Read;     //%00000011
  OutArr[1] := Hi(SRamAddr);   // MSB  A15-A8
  OutArr[2] := Lo(SRamAddr);   // LSB  A7-A0
  OutArr[3] := 0;               //needed?

  InArr[0] := 0;
  InArr[1] := 0;
  InArr[2] := 0;
  InArr[3] := 0;

  ByteCnt    := 4;

  spi.TransferSync(OutArr,InArr,ByteCnt);
  SRAMDataByte := InArr[3];
  Memo1.lines.add('Received Byte : '+IntToStr(SRAMDataByte)+' from '+IntToStr(SRamAddr));
  edtByteValue.caption := IntToStr(SRAMDataByte);
end;

//**********************************************************
//* Procedure : Fills the complete RAM with single Bytes (just to measure the time needed)
//**********************************************************
// This way it needs about 2 seconds to fill the RAM
procedure TForm1.btnByteFillClick(Sender: TObject);
Var
  ByteCnt :  Byte;                 // Nr of Bytes to transmit
  AddressCounter : Word;
  OutArr : Array[0..3] of Byte;
begin
  Memo1.lines.add('Proc : Byte Write');
  btnSetStatusByte.Click;

  OutArr[0] := %00000010;   //Instruction

  For AddressCounter := 0 to 32767 do begin
    OutArr[1] := Hi(AddressCounter);   // MSB  A15-A8
    OutArr[2] := Lo(AddressCounter);   // LSB  A7-A0
    OutArr[3] := spnedtByteValue.value;   // Data to store
    ByteCnt    := 4;
    spi.TransmitSync(OutArr, ByteCnt);
  //  Memo1.lines.add('Stored Byte : '+IntToStr(SRAMDataByte)+' to '+IntToStr(AddressCounter));
  end;
  Memo1.lines.add('Byte Fill finished');
end;

//**********************************************************
//* Procedure : Page Write       1024 Pages of 32 bytes
//**********************************************************

procedure TForm1.btnPageWriteClick(Sender: TObject);
Var
  LocCounter : Byte;
  ByteCount  :  Byte;    // Nr of Bytes to transmit
  OutArr : Array[0..34] of Byte;
begin
  btnSetStatusPage.click;
  Memo1.lines.add('Proc : Page Write');
  PageNum := Lo(spnedtPageNr.value);

  OutArr[0] := %00000010;   //Instruction
  OutArr[1] := Hi(PageNum);   // MSB  A15-A8
  OutArr[2] := Lo(PageNum);   // LSB  A7-A0
  For LocCounter := 0 to 31 do begin
    OutArr[LocCounter +3] := spnedtByteValue2.value;   // Data to store  fill page
    //Memo1.lines.add('Address : '+IntToStr(LocCounter)+' Data : '+IntToStr(OutArr[LocCounter +3]));
    Memo1.lines.add('Page : '+IntToStr(PageNum)+' Address : '+IntToStr(LocCounter-3)+' Data : '+IntToStr(OutArr[LocCounter]));

  end;

  ByteCount    := 35;
  spi.TransmitSync(OutArr, ByteCount);
  Memo1.lines.add('Stored Page : '+IntToStr(PageNum));

end;

//**********************************************************
//* Procedure : Page Read      1024 Pages of 32 bytes
//**********************************************************
procedure TForm1.btnPageReadClick(Sender: TObject);
Var
  LocCounter : Byte;
  ByteCount  :  Byte;    // Nr of Bytes to transmit
  OutArr : Array[0..2] of Byte;
  InArr : Array[0..34] of Byte;
begin
  Memo1.lines.add('Proc :  Page Read');
  btnSetStatusPage.click;

  PageNum := Lo(spnedtPageNr.value);
  OutArr[0] := %00000011;   //Instruction
  OutArr[1] := Hi(PageNum);   // MSB  A15-A8
  OutArr[2] := Lo(PageNum);   // LSB  A7-A0

  For LocCounter := 0 to 34 do begin
    InArr[LocCounter] := 0;   // Erase all values on page
   // Memo1.lines.add('Address : '+IntToStr(LocCounter)+' Data : '+IntToStr(InArr[LocCounter]));
  end;

  ByteCount    := 35;
  spi.TransferSync(OutArr,InArr,ByteCount);
  //spi.TransmitSync(OutArr, ByteCount);

  Memo1.lines.add('Read Page Result : '+IntToStr(ByteCount));
  For LocCounter := 0 to 34 do begin
     // OutArr[LocCounter ] := 0;   // Data to store  fill page
     Memo1.lines.add('Page : '+IntToStr(PageNum)+' Address : '+IntToStr(LocCounter)+' Data : '+IntToStr(InArr[LocCounter]));
   end;
end;

//**********************************************************
//* Procedure : Show Page on Memo1      1024 Pages of 32 bytes
//**********************************************************
procedure TForm1.btnShowPageClick(Sender: TObject);
Var
  LocCounter : Byte;
  ByteCount  :  Byte;    // Nr of Bytes to transmit
  OutArr : Array[0..34] of Byte;
  InArr : Array[0..34] of Byte;
begin
  Memo1.lines.add('Proc : Show Page');
  btnSetStatusPage.click;

  PageNum := Lo(spnedtPageNr.value);
  OutArr[0] := %00000011;   //Instruction
  OutArr[1] := Hi(PageNum);   // MSB  A15-A8
  OutArr[2] := Lo(PageNum);   // LSB  A7-A0

  For LocCounter := 0 to 34 do begin     // Clear Datas
    InArr[LocCounter] := 0;   // Data to store  fill page
    // Memo1.lines.add('Address : '+IntToStr(LocCounter)+' Data : '+IntToStr(InArr[LocCounter]));
  end;

  ByteCount    := 35;
  spi.TransferSync(OutArr,InArr,ByteCount);

  For LocCounter := 3 to 34 do begin     // Show Data on Memo1
    InArr[LocCounter] := 0;   // Data to store  fill page
    Memo1.lines.add('Page : '+IntToStr(PageNum)+' Address : '+IntToStr(LocCounter-3)+' Data : '+IntToStr(InArr[LocCounter]));
  end;
end;

//**********************************************************
//* Procedure : Page Write Fill      1024 Pages of 32 bytes
//**********************************************************
procedure TForm1.btnPageFillClick(Sender: TObject);
Var
  PageNr : Word;
  LocCounter : Byte;
  ByteCount  :  Byte;    // Nr of Bytes to transmit
  OutArr : Array[0..34] of Byte;
begin
  Memo1.lines.add('Proc : Page Write Fill');
  btnSetStatusPage.click;

  OutArr[0] := %00000010;   //Instruction

  For PageNr := 0 to 1023 do Begin

     OutArr[1] := %00000000;   // MSB  A15-A8
     OutArr[2] := %00000000;   // LSB  A7-A0
     For LocCounter := 0 to 31 do begin
       OutArr[LocCounter +3] := LocCounter;   // Data to store  fill page
       // Memo1.lines.add('Address : '+IntToStr(LocCounter)+' Data : '+IntToStr(OutArr[LocCounter +3]));
     end;

     ByteCount    := 35;
     spi.TransmitSync(OutArr, ByteCount);
    // Memo1.lines.add('Stored Page : '+IntToStr(ByteCount));


  End;
  Memo1.lines.add('Page Fill finished');
end;

//**********************************************************
//* Procedure : Sequential Write      32,768 bytes
//**********************************************************
procedure TForm1.btnSeqWriteClick(Sender: TObject);
Var
  DatCnt  : Word;
  ByteCnt : Byte;                 // Nr of Bytes to transmit
  DatArr  : Array[0..32767] of Byte;    // 32768 Bytes  will contain tha data
  OutArr  : Array[0..32770] of Byte;    // 32768 + 3 Bytes -> to send to sram
//  InArr   : Array[0..32771] of Byte;
begin
  Memo1.lines.add('Proc : Sequential Write');
  btnSetStatusSequential.Click;
  OutArr[0] := %00000010;   //Instruction
  OutArr[1] := %00000000;   // MSB  A15-A8
  OutArr[2] := %00000000;   // LSB  A7-A0

  // Daten von DatArr nach OutArr kopieren
  For DatCnt := 0 to 32767 do OutArr[DatCnt+3] := DatArr[DatCnt];

  ByteCnt    := 255;        //  max possible byte value -> problemo
 // ByteCnt    := 32770;    //should be
  spi.TransmitSync(OutArr, ByteCnt);
  Memo1.lines.add('Proc : Sequential Write finished');
end;

//**********************************************************
//* Procedure : Sequential Read      32,768 bytes
//**********************************************************
procedure TForm1.btnSeqReadClick(Sender: TObject);
begin
  btnSetStatusSequential.Click;
 // not implemented now, because problems with sequential write
end;

//**********************************************************
//* Procedure : Set Status To Byte Mode
//**********************************************************
procedure TForm1.btnSetStatusByteClick(Sender: TObject);
Var
  ByteCount  :  Byte;    // Nr of Bytes to transmit
  OutArr : Array[0..1] of Byte;
begin

  If btnSetStatusByte.enabled = true then begin
    Memo1.lines.add('Proc : Set to Byte Mode');
    btnSetStatusByte.enabled := False;
    btnSetStatusPage.enabled := True;
    btnSetStatusSequential.enabled := True;

    OutArr[0] := %00000001;   //Instruction
    OutArr[1] := %00000001;   // Data to Status Register

    ByteCount    := 2;
    spi.TransmitSync(OutArr, ByteCount);
  End;


end;



//**********************************************************
//* Procedure : Set Status To Page Mode
//**********************************************************
procedure TForm1.btnSetStatusPageClick(Sender: TObject);
Var
  ByteCount  :  Byte;    // Nr of Bytes to transmit
  OutArr : Array[0..1] of Byte;
begin
  If btnSetStatusPage.enabled = True then begin
    Memo1.lines.add('Proc : Set to Page Mode');

    btnSetStatusByte.enabled := True;
    btnSetStatusPage.enabled := False;
    btnSetStatusSequential.enabled := True;

    OutArr[0] := %00000001;   //Instruction
    OutArr[1] := %10000001;   // Data to Status Register

    ByteCount    := 2;
    spi.TransmitSync(OutArr, ByteCount);
  end;
end;

//**********************************************************
//* Procedure : Set Status To Sequential Mode
//**********************************************************
procedure TForm1.btnSetStatusSequentialClick(Sender: TObject);
Var
  ByteCount  :  Byte;    // Nr of Bytes to transmit
  OutArr : Array[0..1] of Byte;
begin
  If   btnSetStatusSequential.enabled = True then Begin
    Memo1.lines.add('Proc : Set to Sequential Mode');

    btnSetStatusByte.enabled := True;
    btnSetStatusPage.enabled := True;
    btnSetStatusSequential.enabled := False;

    OutArr[0] := %00000001;   //Instruction
    OutArr[1] := %01000001;   // Data to Status Register

    ByteCount    := 2;
    spi.TransmitSync(OutArr, ByteCount);

  end;
end;

//**********************************************************
//* Procedure : Read Status Register
//**********************************************************
procedure TForm1.btnReadStatusClick(Sender: TObject);
//Sends one byte and receives one Byte (Status)
Var
  ByteCnt :  Byte;                 // Nr of Bytes to transmit
  OutArr  : Array[0..1] of Byte;
  InArr   : Array[0..1] of Byte;

begin

  OutArr[0] := Instr_RDSR;     //%00000101
  OutArr[1] := 0;              //needed?

  InArr[0] := 0;
  InArr[1] := 0;

  ByteCnt    := 2;
  spi.TransferSync(OutArr,InArr,ByteCnt);

  Case InArr[1] of
      1: Memo1.lines.add('Status : '+IntToStr(InArr[1])+' Byte Mode');
     65: Memo1.lines.add('Status : '+IntToStr(InArr[1])+' Sequential Mode');
    129: Memo1.lines.add('Status : '+IntToStr(InArr[1])+' Page Mode');
    193: Memo1.lines.add('Status : '+IntToStr(InArr[1])+' reserved');
    Else Memo1.lines.add('Status : '+IntToStr(InArr[1])+' unknown'+' Len : '+IntToStr(ByteCnt));
  end;
  Memo1.lines.add('Address : '+IntToStr(0)+' Data : '+IntToStr(InArr[0]));
  Memo1.lines.add('Address : '+IntToStr(1)+' Data : '+IntToStr(InArr[1]));

end;

//**********************************************************
//* Procedure : Set SPI Device 0
//**********************************************************
procedure TForm1.rbtnSPIDev0Change(Sender: TObject);
begin
  If rbtnSPIDev0.checked then begin
    cSPI_DEVICE := '/dev/spidev0.0';  // Device 0, ChipSelect 0
    Memo1.lines.add('SPI Device : '+cSPI_DEVICE);
  end;
end;

//**********************************************************
//* Procedure : Set SPI Device 1
//**********************************************************
procedure TForm1.rbtnSPIDev1Change(Sender: TObject);
begin
  // Umschalten läuft noch nicht.
  //  cSPI_DEVICE  funktioniert nur als konstante
  If rbtnSPIDev1.checked then begin
    cSPI_DEVICE := '/dev/spidev0.1';  // Device 0, ChipSelect 1
    Memo1.lines.add('SPI Device : '+cSPI_DEVICE);
  end;
end;

procedure TForm1.spnedtAddressBrowseChange(Sender: TObject);
begin
  btnSetStatusByte.Click;
  spnedtAddress.value := spnedtAddressBrowse.value;
 // SRamAddr := Lo(spnedtAddress.value);
  //Memo1.lines.add('Hi Byte : '+IntToStr(Hi(SRamAddr))+' Lo Byte : '+IntToStr(Lo(SRamAddr)));
  btnByteRead.Click;

end;

procedure TForm1.spnedtAddressChange(Sender: TObject);
begin
  btnSetStatusByte.Click;
  SRamAddr := Lo(spnedtAddress.value);
  // Memo1.lines.add('Hi Byte : '+IntToStr(Hi(SRamAddr))+' Lo Byte : '+IntToStr(Lo(SRamAddr)));
end;

procedure TForm1.spnedtByteValue2Change(Sender: TObject);
begin
  btnSetStatusPage.click;
end;

procedure TForm1.spnedtByteValueChange(Sender: TObject);
begin
  btnSetStatusByte.Click;
  SRamDataByte := spnedtByteValue.Value;
  Memo1.lines.add('Data Byte : '+IntToStr(SRAMDataByte));
end;

procedure TForm1.spnedtPageNrChange(Sender: TObject);
begin
  btnSetStatusPage.click;
  PageNum := Lo(spnedtPageNr.value);
  Memo1.lines.add('Page Nr : '+IntToStr(PageNum)+' Bin Nr : '+IntToBin(PageNum,16,8));
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
  spi.Init();
  cSPI_DEVICE := '/dev/spidev0.0';  // Device 0, ChipSelect 0
  btnSetStatusByte.enabled := True;
  btnSetStatusPage.enabled := True;
  btnSetStatusSequential.enabled := True;
  Memo1.SelStart := -2;
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
  spi.Close();
end;

end.

